@@ -19,14 +19,11 @@
 
 package org.apache.hudi.cli.commands;
 
-import org.apache.hudi.cli.HoodieCLI;
 import org.apache.hudi.cli.HoodiePrintHelper;
 import org.apache.hudi.cli.HoodieTableHeaderFields;
+import org.apache.hudi.cli.utils.CLIUtils;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.model.HoodieWriteStat;
-import org.apache.hudi.common.table.HoodieTableMetaClient;
-import org.apache.hudi.common.table.timeline.HoodieActiveTimeline;
-import org.apache.hudi.common.table.timeline.HoodieArchivedTimeline;
 import org.apache.hudi.common.table.timeline.HoodieDefaultTimeline;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.util.NumericUtils;
@@ -45,11 +42,6 @@ import java.util.function.BiFunction;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
-import static org.apache.hudi.cli.utils.CommitUtil.getTimeDaysAgo;
-import static org.apache.hudi.common.util.StringUtils.isNullOrEmpty;
-import static org.apache.hudi.common.util.StringUtils.nonEmpty;
-import static org.apache.hudi.common.util.ValidationUtils.checkArgument;
-
 /**
  * Given a file id or partition value, this command line utility tracks the changes to the file group or partition across range of commits.
  * Usage: diff file --fileId <fileId>
@@ -64,16 +56,16 @@ public class DiffCommand {
   public String diffFile(
       @ShellOption(value = {"--fileId"}, help = "File ID to diff across range of commits") String fileId,
       @ShellOption(value = {"--startTs"}, help = "start time for compactions, default: now - 10 days",
-              defaultValue = ShellOption.NULL) String startTs,
+          defaultValue = ShellOption.NULL) String startTs,
       @ShellOption(value = {"--endTs"}, help = "end time for compactions, default: now - 1 day",
-              defaultValue = ShellOption.NULL) String endTs,
+          defaultValue = ShellOption.NULL) String endTs,
       @ShellOption(value = {"--limit"}, help = "Limit compactions", defaultValue = "-1") final Integer limit,
       @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
       @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
       @ShellOption(value = {"--headeronly"}, help = "Print Header Only", defaultValue = "false") final boolean headerOnly,
       @ShellOption(value = {"--includeArchivedTimeline"}, help = "Include archived commits as well",
           defaultValue = "false") final boolean includeArchivedTimeline) throws IOException {
-    HoodieDefaultTimeline timeline = getTimelineInRange(startTs, endTs, includeArchivedTimeline);
+    HoodieDefaultTimeline timeline = CLIUtils.getTimelineInRange(startTs, endTs, includeArchivedTimeline);
     return printCommitsWithMetadataForFileId(timeline, limit, sortByField, descending, headerOnly, "", fileId);
   }
 
@@ -81,38 +73,19 @@ public class DiffCommand {
   public String diffPartition(
       @ShellOption(value = {"--partitionPath"}, help = "Relative partition path to diff across range of commits") String partitionPath,
       @ShellOption(value = {"--startTs"}, help = "start time for compactions, default: now - 10 days",
-              defaultValue = ShellOption.NULL) String startTs,
+          defaultValue = ShellOption.NULL) String startTs,
       @ShellOption(value = {"--endTs"}, help = "end time for compactions, default: now - 1 day",
-              defaultValue = ShellOption.NULL) String endTs,
+          defaultValue = ShellOption.NULL) String endTs,
       @ShellOption(value = {"--limit"}, help = "Limit compactions", defaultValue = "-1") final Integer limit,
       @ShellOption(value = {"--sortBy"}, help = "Sorting Field", defaultValue = "") final String sortByField,
       @ShellOption(value = {"--desc"}, help = "Ordering", defaultValue = "false") final boolean descending,
       @ShellOption(value = {"--headeronly"}, help = "Print Header Only", defaultValue = "false") final boolean headerOnly,
       @ShellOption(value = {"--includeArchivedTimeline"}, help = "Include archived commits as well",
           defaultValue = "false") final boolean includeArchivedTimeline) throws IOException {
-    HoodieDefaultTimeline timeline = getTimelineInRange(startTs, endTs, includeArchivedTimeline);
+    HoodieDefaultTimeline timeline = CLIUtils.getTimelineInRange(startTs, endTs, includeArchivedTimeline);
     return printCommitsWithMetadataForPartition(timeline, limit, sortByField, descending, headerOnly, "", partitionPath);
   }
 
-  private HoodieDefaultTimeline getTimelineInRange(String startTs, String endTs, boolean includeArchivedTimeline) {
-    if (isNullOrEmpty(startTs)) {
-      startTs = getTimeDaysAgo(10);
-    }
-    if (isNullOrEmpty(endTs)) {
-      endTs = getTimeDaysAgo(1);
-    }
-    checkArgument(nonEmpty(startTs), "startTs is null or empty");
-    checkArgument(nonEmpty(endTs), "endTs is null or empty");
-    HoodieTableMetaClient metaClient = HoodieCLI.getTableMetaClient();
-    HoodieActiveTimeline activeTimeline = metaClient.getActiveTimeline();
-    if (includeArchivedTimeline) {
-      HoodieArchivedTimeline archivedTimeline = metaClient.getArchivedTimeline();
-      archivedTimeline.loadInstantDetailsInMemory(startTs, endTs);
-      return archivedTimeline.findInstantsInRange(startTs, endTs).mergeTimeline(activeTimeline);
-    }
-    return activeTimeline;
-  }
-
   private String printCommitsWithMetadataForFileId(HoodieDefaultTimeline timeline,
                                                    final Integer limit,
                                                    final String sortByField,
