@@ -26,8 +26,8 @@ import org.apache.hudi.table.HoodieTable;
 
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 
@@ -38,7 +38,7 @@ import static org.apache.hudi.common.heartbeat.HoodieHeartbeatUtils.getLastHeart
  */
 public class HeartbeatUtils {
 
-  private static final Logger LOG = LogManager.getLogger(HeartbeatUtils.class);
+  private static final Logger LOG = LoggerFactory.getLogger(HeartbeatUtils.class);
 
   /**
    * Deletes the heartbeat file for the specified instant.
