@@ -20,6 +20,7 @@ package org.apache.hudi.io;
 
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.model.HoodieKey;
+import org.apache.hudi.common.model.HoodieRecordGlobalLocation;
 import org.apache.hudi.common.model.HoodieRecordLocation;
 import org.apache.hudi.common.util.BaseFileUtils;
 import org.apache.hudi.common.util.Option;
@@ -30,7 +31,6 @@ import org.apache.hudi.table.HoodieTable;
 
 import org.apache.hadoop.fs.Path;
 
-import java.util.ArrayList;
 import java.util.List;
 import java.util.stream.Stream;
 
@@ -51,17 +51,28 @@ public class HoodieKeyLocationFetchHandle<T, I, K, O> extends HoodieReadHandle<T
     this.keyGeneratorOpt = keyGeneratorOpt;
   }
 
+  private List<HoodieKey> fetchHoodieKeys(HoodieBaseFile baseFile) {
+    BaseFileUtils baseFileUtils = BaseFileUtils.getInstance(baseFile.getPath());
+    if (keyGeneratorOpt.isPresent()) {
+      return baseFileUtils.fetchHoodieKeys(hoodieTable.getHadoopConf(), new Path(baseFile.getPath()), keyGeneratorOpt);
+    } else {
+      return baseFileUtils.fetchHoodieKeys(hoodieTable.getHadoopConf(), new Path(baseFile.getPath()));
+    }
+  }
+
   public Stream<Pair<HoodieKey, HoodieRecordLocation>> locations() {
     HoodieBaseFile baseFile = partitionPathBaseFilePair.getRight();
-    BaseFileUtils baseFileUtils = BaseFileUtils.getInstance(baseFile.getPath());
-    List<HoodieKey> hoodieKeyList = new ArrayList<>();
-    if (keyGeneratorOpt.isPresent()) {
-      hoodieKeyList = baseFileUtils.fetchHoodieKeys(hoodieTable.getHadoopConf(), new Path(baseFile.getPath()), keyGeneratorOpt);
-    } else {
-      hoodieKeyList = baseFileUtils.fetchHoodieKeys(hoodieTable.getHadoopConf(), new Path(baseFile.getPath()));
-    }
-    return hoodieKeyList.stream()
+    String commitTime = baseFile.getCommitTime();
+    String fileId = baseFile.getFileId();
+    return fetchHoodieKeys(baseFile).stream()
         .map(entry -> Pair.of(entry,
-            new HoodieRecordLocation(baseFile.getCommitTime(), baseFile.getFileId())));
+            new HoodieRecordLocation(commitTime, fileId)));
+  }
+
+  public Stream<Pair<String, HoodieRecordGlobalLocation>> globalLocations() {
+    HoodieBaseFile baseFile = partitionPathBaseFilePair.getRight();
+    return fetchHoodieKeys(baseFile).stream()
+        .map(entry -> Pair.of(entry.getRecordKey(),
+            new HoodieRecordGlobalLocation(entry.getPartitionPath(), baseFile.getCommitTime(), baseFile.getFileId())));
   }
 }
