@@ -79,6 +79,12 @@ public abstract class PartitionAwareClusteringPlanStrategy<T,I,K,O> extends Clus
         fileSliceGroups.add(Pair.of(currentGroup, numOutputGroups));
         currentGroup = new ArrayList<>();
         totalSizeSoFar = 0;
+
+        // if fileSliceGroups's size reach the max group, stop loop
+        if (fileSliceGroups.size() >= writeConfig.getClusteringMaxNumGroups()) {
+          LOG.info("Having generated the maximum number of groups : " + writeConfig.getClusteringMaxNumGroups());
+          break;
+        }
       }
 
       // Add to the current file-group
