@@ -18,18 +18,6 @@
 
 package org.apache.hudi.table.action.rollback;
 
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.List;
-import java.util.Map;
-import java.util.function.Function;
-import java.util.stream.Collectors;
-import org.apache.hadoop.fs.FileStatus;
-import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.fs.PathFilter;
 import org.apache.hudi.avro.model.HoodieRollbackRequest;
 import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.fs.FSUtils;
@@ -47,9 +35,24 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieRollbackException;
 import org.apache.hudi.table.HoodieTable;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
+
+import org.apache.hadoop.fs.FileStatus;
+import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.fs.Path;
+import org.apache.hadoop.fs.PathFilter;
 import org.jetbrains.annotations.NotNull;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
+import java.util.Map;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.function.Function;
+import java.util.stream.Collectors;
 
 import static org.apache.hudi.client.utils.MetadataConversionUtils.getHoodieCommitMetadata;
 import static org.apache.hudi.common.util.ValidationUtils.checkArgument;
@@ -60,7 +63,7 @@ import static org.apache.hudi.table.action.rollback.BaseRollbackHelper.EMPTY_STR
  */
 public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecutor.RollbackStrategy {
 
-  private static final Logger LOG = LogManager.getLogger(ListingBasedRollbackStrategy.class);
+  private static final Logger LOG = LoggerFactory.getLogger(ListingBasedRollbackStrategy.class);
 
   protected final HoodieTable<?, ?, ?, ?> table;
 
@@ -70,14 +73,18 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
 
   protected final String instantTime;
 
+  protected final Boolean isRestore;
+
   public ListingBasedRollbackStrategy(HoodieTable<?, ?, ?, ?> table,
                                       HoodieEngineContext context,
                                       HoodieWriteConfig config,
-                                      String instantTime) {
+                                      String instantTime,
+                                      boolean isRestore) {
     this.table = table;
     this.context = context;
     this.config = config;
     this.instantTime = instantTime;
+    this.isRestore = isRestore;
   }
 
   @Override
@@ -94,43 +101,47 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
       String baseFileExtension = getBaseFileExtension(metaClient);
       Option<HoodieCommitMetadata> commitMetadataOptional = getHoodieCommitMetadata(metaClient, instantToRollback);
       Boolean isCommitMetadataCompleted = checkCommitMetadataCompleted(instantToRollback, commitMetadataOptional);
+      AtomicBoolean isCompaction = new AtomicBoolean(false);
+      if (commitMetadataOptional.isPresent()) {
+        isCompaction.set(commitMetadataOptional.get().getOperationType() == WriteOperationType.COMPACT);
+      }
 
       return context.flatMap(partitionPaths, partitionPath -> {
         List<HoodieRollbackRequest> hoodieRollbackRequests = new ArrayList<>(partitionPaths.size());
         FileStatus[] filesToDelete =
             fetchFilesFromInstant(instantToRollback, partitionPath, metaClient.getBasePath(), baseFileExtension,
-                metaClient.getFs(), commitMetadataOptional, isCommitMetadataCompleted);
+                metaClient.getFs(), commitMetadataOptional, isCommitMetadataCompleted, tableType);
 
         if (HoodieTableType.COPY_ON_WRITE == tableType) {
-          hoodieRollbackRequests.add(getHoodieRollbackRequest(partitionPath, filesToDelete));
+          hoodieRollbackRequests.addAll(getHoodieRollbackRequests(partitionPath, filesToDelete));
         } else if (HoodieTableType.MERGE_ON_READ == tableType) {
           String commit = instantToRollback.getTimestamp();
           HoodieActiveTimeline activeTimeline = table.getMetaClient().reloadActiveTimeline();
-          switch (instantToRollback.getAction()) {
+          String action = instantToRollback.getAction();
+          if (isCompaction.get()) { // compaction's action in hoodie instant will be "commit". So, we might need to override.
+            action = HoodieTimeline.COMPACTION_ACTION;
+          }
+          switch (action) {
             case HoodieTimeline.COMMIT_ACTION:
             case HoodieTimeline.REPLACE_COMMIT_ACTION:
-              hoodieRollbackRequests.add(getHoodieRollbackRequest(partitionPath, filesToDelete));
+              hoodieRollbackRequests.addAll(getHoodieRollbackRequests(partitionPath, filesToDelete));
               break;
             case HoodieTimeline.COMPACTION_ACTION:
-              // If there is no delta commit present after the current commit (if compaction), no action, else we
-              // need to make sure that a compaction commit rollback also deletes any log files written as part of the
-              // succeeding deltacommit.
-              boolean higherDeltaCommits =
-                  !activeTimeline.getDeltaCommitTimeline().filterCompletedInstants().findInstantsAfter(commit, 1)
-                      .empty();
-              if (higherDeltaCommits) {
-                // Rollback of a compaction action with no higher deltacommit means that the compaction is scheduled
+              // Depending on whether we are rolling back compaction as part of restore or a regular rollback, logic differs/
+              // as part of regular rollback(on re-attempting a failed compaction), we might have to delete/rollback only the base file that could have
+              // potentially been created. Even if there are log files added to the file slice of interest, we should not touch them.
+              // but if its part of a restore operation, rolling back a compaction should rollback entire file slice, i.e base file and all log files.
+              if (!isRestore) {
+                // Rollback of a compaction action if not for restore means that the compaction is scheduled
                 // and has not yet finished. In this scenario we should delete only the newly created base files
                 // and not corresponding base commit log files created with this as baseCommit since updates would
                 // have been written to the log files.
-                hoodieRollbackRequests.add(getHoodieRollbackRequest(partitionPath,
-                    listFilesToBeDeleted(instantToRollback.getTimestamp(), baseFileExtension, partitionPath,
+                hoodieRollbackRequests.addAll(getHoodieRollbackRequests(partitionPath,
+                    listBaseFilesToBeDeleted(instantToRollback.getTimestamp(), baseFileExtension, partitionPath,
                         metaClient.getFs())));
               } else {
-                // No deltacommits present after this compaction commit (inflight or requested). In this case, we
-                // can also delete any log files that were created with this compaction commit as base
-                // commit.
-                hoodieRollbackRequests.add(getHoodieRollbackRequest(partitionPath, filesToDelete));
+                // if this is part of a restore operation, we should rollback/delete entire file slice.
+                hoodieRollbackRequests.addAll(getHoodieRollbackRequests(partitionPath, filesToDelete));
               }
               break;
             case HoodieTimeline.DELTA_COMMIT_ACTION:
@@ -142,7 +153,7 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
               // (A.2) Failed recurring commit - Inserts/Updates written to log files. In this scenario,
               // HoodieWriteStat will have the baseCommitTime for the first log file written, add rollback blocks.
               // (A.3) Rollback triggered for first commit - Inserts were written to the log files but the commit is
-              // being reverted. In this scenario, HoodieWriteStat will be `null` for the attribute prevCommitTime and
+              // being reverted. In this scenario, HoodieWriteStat will be `null` for the attribute prevCommitTime
               // and hence will end up deleting these log files. This is done so there are no orphan log files
               // lying around.
               // (A.4) Rollback triggered for recurring commits - Inserts/Updates are being rolled back, the actions
@@ -156,7 +167,7 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
               // this scenario, perform (A.1) and for updates written to log files, write rollback blocks.
               // (B.3) Rollback triggered for first commit - Same as (B.1)
               // (B.4) Rollback triggered for recurring commits - Same as (B.2) plus we need to delete the log files
-              // as well if the base base file gets deleted.
+              // as well if the base file gets deleted.
               HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(
                   table.getMetaClient().getCommitTimeline().getInstantDetails(instantToRollback).get(),
                   HoodieCommitMetadata.class);
@@ -164,7 +175,7 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
               // In case all data was inserts and the commit failed, delete the file belonging to that commit
               // We do not know fileIds for inserts (first inserts are either log files or base files),
               // delete all files for the corresponding failed commit, if present (same as COW)
-              hoodieRollbackRequests.add(getHoodieRollbackRequest(partitionPath, filesToDelete));
+              hoodieRollbackRequests.addAll(getHoodieRollbackRequests(partitionPath, filesToDelete));
 
               // append rollback blocks for updates and inserts as A.2 and B.2
               if (commitMetadata.getPartitionToWriteStats().containsKey(partitionPath)) {
@@ -192,23 +203,19 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
   }
 
   @NotNull
-  private HoodieRollbackRequest getHoodieRollbackRequest(String partitionPath, FileStatus[] filesToDeletedStatus) {
-    List<String> filesToDelete = getFilesToBeDeleted(filesToDeletedStatus);
-    return new HoodieRollbackRequest(
-        partitionPath, EMPTY_STRING, EMPTY_STRING, filesToDelete, Collections.emptyMap());
+  private List<HoodieRollbackRequest> getHoodieRollbackRequests(String partitionPath, FileStatus[] filesToDeletedStatus) {
+    return Arrays.stream(filesToDeletedStatus)
+        .map(fileStatus -> {
+          String dataFileToBeDeleted = fileStatus.getPath().toString();
+          // strip scheme E.g: file:/var/folders
+          return dataFileToBeDeleted.substring(dataFileToBeDeleted.indexOf(":") + 1);
+        })
+        .map(s -> new HoodieRollbackRequest(partitionPath, EMPTY_STRING, EMPTY_STRING, Collections.singletonList(s), Collections.emptyMap()))
+        .collect(Collectors.toList());
   }
 
-  @NotNull
-  private List<String> getFilesToBeDeleted(FileStatus[] dataFilesToDeletedStatus) {
-    return Arrays.stream(dataFilesToDeletedStatus).map(fileStatus -> {
-      String dataFileToBeDeleted = fileStatus.getPath().toString();
-      // strip scheme E.g: file:/var/folders
-      return dataFileToBeDeleted.substring(dataFileToBeDeleted.indexOf(":") + 1);
-    }).collect(Collectors.toList());
-  }
-
-  private FileStatus[] listFilesToBeDeleted(String commit, String basefileExtension, String partitionPath,
-                                            FileSystem fs) throws IOException {
+  private FileStatus[] listBaseFilesToBeDeleted(String commit, String basefileExtension, String partitionPath,
+                                                FileSystem fs) throws IOException {
     LOG.info("Collecting files to be cleaned/rolledback up for path " + partitionPath + " and commit " + commit);
     PathFilter filter = (path) -> {
       if (path.toString().contains(basefileExtension)) {
@@ -223,8 +230,10 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
   private FileStatus[] fetchFilesFromInstant(HoodieInstant instantToRollback, String partitionPath, String basePath,
                                              String baseFileExtension, HoodieWrapperFileSystem fs,
                                              Option<HoodieCommitMetadata> commitMetadataOptional,
-                                             Boolean isCommitMetadataCompleted) throws IOException {
-    if (isCommitMetadataCompleted) {
+                                             Boolean isCommitMetadataCompleted,
+                                             HoodieTableType tableType) throws IOException {
+    // go w/ commit metadata only for COW table. for MOR, we need to get associated log files when commit corresponding to base file is rolledback.
+    if (isCommitMetadataCompleted && tableType == HoodieTableType.COPY_ON_WRITE) {
       return fetchFilesFromCommitMetadata(instantToRollback, partitionPath, basePath, commitMetadataOptional.get(),
           baseFileExtension, fs);
     } else {
@@ -250,6 +259,16 @@ public class ListingBasedRollbackStrategy implements BaseRollbackPlanActionExecu
     }).toArray(Path[]::new), pathFilter);
   }
 
+  /**
+   * returns matching base files and log files if any for the instant time of the commit to be rolled back.
+   * @param instantToRollback
+   * @param partitionPath
+   * @param basePath
+   * @param baseFileExtension
+   * @param fs
+   * @return
+   * @throws IOException
+   */
   private FileStatus[] fetchFilesFromListFiles(HoodieInstant instantToRollback, String partitionPath, String basePath,
                                                String baseFileExtension, HoodieWrapperFileSystem fs)
       throws IOException {
