@@ -81,7 +81,7 @@ public class HoodieMetadataTestTable extends HoodieTestTable {
         partitionToFilesNameLengthMap, bootstrap, createInflightCommit);
     if (writer != null && !createInflightCommit) {
       writer.performTableServices(Option.of(commitTime));
-      writer.update(commitMetadata, context.get().emptyHoodieData(), commitTime);
+      writer.updateFromWriteStatuses(commitMetadata, context.get().emptyHoodieData(), commitTime);
     }
     return commitMetadata;
   }
@@ -90,7 +90,7 @@ public class HoodieMetadataTestTable extends HoodieTestTable {
   public HoodieTestTable moveInflightCommitToComplete(String instantTime, HoodieCommitMetadata metadata) throws IOException {
     super.moveInflightCommitToComplete(instantTime, metadata);
     if (writer != null) {
-      writer.update(metadata, context.get().emptyHoodieData(), instantTime);
+      writer.updateFromWriteStatuses(metadata, context.get().emptyHoodieData(), instantTime);
     }
     return this;
   }
@@ -98,7 +98,7 @@ public class HoodieMetadataTestTable extends HoodieTestTable {
   public HoodieTestTable moveInflightCommitToComplete(String instantTime, HoodieCommitMetadata metadata, boolean ignoreWriter) throws IOException {
     super.moveInflightCommitToComplete(instantTime, metadata);
     if (!ignoreWriter && writer != null) {
-      writer.update(metadata, context.get().emptyHoodieData(), instantTime);
+      writer.updateFromWriteStatuses(metadata, context.get().emptyHoodieData(), instantTime);
     }
     return this;
   }
@@ -107,7 +107,7 @@ public class HoodieMetadataTestTable extends HoodieTestTable {
   public HoodieTestTable moveInflightCompactionToComplete(String instantTime, HoodieCommitMetadata metadata) throws IOException {
     super.moveInflightCompactionToComplete(instantTime, metadata);
     if (writer != null) {
-      writer.update(metadata, context.get().emptyHoodieData(), instantTime);
+      writer.updateFromWriteStatuses(metadata, context.get().emptyHoodieData(), instantTime);
     }
     return this;
   }
@@ -124,7 +124,7 @@ public class HoodieMetadataTestTable extends HoodieTestTable {
   public HoodieTestTable addCompaction(String instantTime, HoodieCommitMetadata commitMetadata) throws Exception {
     super.addCompaction(instantTime, commitMetadata);
     if (writer != null) {
-      writer.update(commitMetadata, context.get().emptyHoodieData(), instantTime);
+      writer.updateFromWriteStatuses(commitMetadata, context.get().emptyHoodieData(), instantTime);
     }
     return this;
   }
@@ -156,7 +156,7 @@ public class HoodieMetadataTestTable extends HoodieTestTable {
       HoodieReplaceCommitMetadata completeReplaceMetadata) throws Exception {
     super.addReplaceCommit(instantTime, requestedReplaceMetadata, inflightReplaceMetadata, completeReplaceMetadata);
     if (writer != null) {
-      writer.update(completeReplaceMetadata, context.get().emptyHoodieData(), instantTime);
+      writer.updateFromWriteStatuses(completeReplaceMetadata, context.get().emptyHoodieData(), instantTime);
     }
     return this;
   }
