@@ -18,13 +18,21 @@
 
 package org.apache.hudi.index;
 
-import org.apache.hudi.common.util.ReflectionUtils;
+import org.apache.hudi.common.config.TypedProperties;
+import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.StringUtils;
 import org.apache.hudi.config.HoodieWriteConfig;
+import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.exception.HoodieIndexException;
 import org.apache.hudi.index.bloom.HoodieBloomIndex;
 import org.apache.hudi.index.bloom.ListBasedHoodieBloomIndexHelper;
 import org.apache.hudi.index.inmemory.HoodieInMemoryHashIndex;
+import org.apache.hudi.index.simple.HoodieGlobalSimpleIndex;
+import org.apache.hudi.index.simple.HoodieSimpleIndex;
+import org.apache.hudi.keygen.BaseKeyGenerator;
+import org.apache.hudi.keygen.factory.HoodieAvroKeyGeneratorFactory;
+
+import java.io.IOException;
 
 /**
  * A factory to generate Java {@link HoodieIndex}.
@@ -34,15 +42,15 @@ public final class JavaHoodieIndexFactory {
   public static HoodieIndex createIndex(HoodieWriteConfig config) {
     // first use index class config to create index.
     if (!StringUtils.isNullOrEmpty(config.getIndexClass())) {
-      Object instance = ReflectionUtils.loadClass(config.getIndexClass(), config);
-      if (!(instance instanceof HoodieIndex)) {
-        throw new HoodieIndexException(config.getIndexClass() + " is not a subclass of HoodieIndex");
-      }
-      return (HoodieIndex) instance;
+      return HoodieIndexUtils.createUserDefinedIndex(config);
     }
 
     // TODO more indexes to be added
     switch (config.getIndexType()) {
+      case SIMPLE:
+        return new HoodieSimpleIndex(config, getKeyGeneratorForSimpleIndex(config));
+      case GLOBAL_SIMPLE:
+        return new HoodieGlobalSimpleIndex(config, getKeyGeneratorForSimpleIndex(config));
       case INMEMORY:
         return new HoodieInMemoryHashIndex(config);
       case BLOOM:
@@ -51,4 +59,13 @@ public final class JavaHoodieIndexFactory {
         throw new HoodieIndexException("Unsupported index type " + config.getIndexType());
     }
   }
+
+  private static Option<BaseKeyGenerator> getKeyGeneratorForSimpleIndex(HoodieWriteConfig config) {
+    try {
+      return config.populateMetaFields() ? Option.empty()
+          : Option.of((BaseKeyGenerator) HoodieAvroKeyGeneratorFactory.createKeyGenerator(new TypedProperties(config.getProps())));
+    } catch (IOException e) {
+      throw new HoodieIOException("KeyGenerator instantiation failed ", e);
+    }
+  }
 }
