@@ -18,14 +18,18 @@
 
 package org.apache.hudi.io.storage.row;
 
-import org.apache.hudi.client.HoodieInternalWriteStatus;
+import org.apache.hudi.client.WriteStatus;
 import org.apache.hudi.client.model.HoodieInternalRow;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodiePartitionMetadata;
 import org.apache.hudi.common.model.HoodieRecord;
+import org.apache.hudi.common.model.HoodieRecordDelegate;
+import org.apache.hudi.common.model.HoodieRecordLocation;
 import org.apache.hudi.common.model.HoodieWriteStat;
 import org.apache.hudi.common.model.IOType;
 import org.apache.hudi.common.util.HoodieTimer;
+import org.apache.hudi.common.util.Option;
+import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieException;
 import org.apache.hudi.exception.HoodieIOException;
@@ -36,11 +40,11 @@ import org.apache.hudi.table.marker.WriteMarkersFactory;
 
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
 import org.apache.spark.sql.catalyst.InternalRow;
 import org.apache.spark.sql.types.StructType;
 import org.apache.spark.unsafe.types.UTF8String;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.io.Serializable;
@@ -54,7 +58,7 @@ public class HoodieRowCreateHandle implements Serializable {
 
   private static final long serialVersionUID = 1L;
 
-  private static final Logger LOG = LogManager.getLogger(HoodieRowCreateHandle.class);
+  private static final Logger LOG = LoggerFactory.getLogger(HoodieRowCreateHandle.class);
   private static final AtomicLong GLOBAL_SEQ_NO = new AtomicLong(1);
 
   private final HoodieTable table;
@@ -70,10 +74,13 @@ public class HoodieRowCreateHandle implements Serializable {
   private final UTF8String commitTime;
   private final Function<Long, String> seqIdGenerator;
 
+  private final boolean shouldPreserveHoodieMetadata;
+
   private final HoodieTimer currTimer;
 
   protected final HoodieInternalRowFileWriter fileWriter;
-  protected final HoodieInternalWriteStatus writeStatus;
+  protected final WriteStatus writeStatus;
+  private final HoodieRecordLocation newRecordLocation;
 
   public HoodieRowCreateHandle(HoodieTable table,
                                HoodieWriteConfig writeConfig,
@@ -84,10 +91,25 @@ public class HoodieRowCreateHandle implements Serializable {
                                long taskId,
                                long taskEpochId,
                                StructType structType) {
+    this(table, writeConfig, partitionPath, fileId, instantTime, taskPartitionId, taskId, taskEpochId,
+        structType, false);
+  }
+
+  public HoodieRowCreateHandle(HoodieTable table,
+                               HoodieWriteConfig writeConfig,
+                               String partitionPath,
+                               String fileId,
+                               String instantTime,
+                               int taskPartitionId,
+                               long taskId,
+                               long taskEpochId,
+                               StructType structType,
+                               boolean shouldPreserveHoodieMetadata) {
     this.partitionPath = partitionPath;
     this.table = table;
     this.writeConfig = writeConfig;
     this.fileId = fileId;
+    this.newRecordLocation = new HoodieRecordLocation(instantTime, fileId);
 
     this.currTimer = HoodieTimer.start();
 
@@ -102,8 +124,10 @@ public class HoodieRowCreateHandle implements Serializable {
     this.commitTime = UTF8String.fromString(instantTime);
     this.seqIdGenerator = (id) -> HoodieRecord.generateSequenceId(instantTime, taskPartitionId, id);
 
-    this.writeStatus = new HoodieInternalWriteStatus(!table.getIndex().isImplicitWithStorage(),
+    this.writeStatus = new WriteStatus(table.shouldTrackSuccessRecords(),
         writeConfig.getWriteStatusFailureFraction());
+    this.shouldPreserveHoodieMetadata = shouldPreserveHoodieMetadata;
+
     writeStatus.setPartitionPath(partitionPath);
     writeStatus.setFileId(fileId);
     writeStatus.setStat(new HoodieWriteStat());
@@ -154,19 +178,23 @@ public class HoodieRowCreateHandle implements Serializable {
       UTF8String recordKey = row.getUTF8String(HoodieRecord.RECORD_KEY_META_FIELD_ORD);
       UTF8String partitionPath = row.getUTF8String(HoodieRecord.PARTITION_PATH_META_FIELD_ORD);
       // This is the only meta-field that is generated dynamically, hence conversion b/w
-      // [[String]] and [[UTF8String]] is unavoidable
-      UTF8String seqId = UTF8String.fromString(seqIdGenerator.apply(GLOBAL_SEQ_NO.getAndIncrement()));
+      // [[String]] and [[UTF8String]] is unavoidable if preserveHoodieMetadata is false
+      UTF8String seqId = shouldPreserveHoodieMetadata ? row.getUTF8String(HoodieRecord.COMMIT_SEQNO_METADATA_FIELD_ORD)
+          : UTF8String.fromString(seqIdGenerator.apply(GLOBAL_SEQ_NO.getAndIncrement()));
+      UTF8String writeCommitTime = shouldPreserveHoodieMetadata ? row.getUTF8String(HoodieRecord.COMMIT_TIME_METADATA_FIELD_ORD)
+          : commitTime;
 
-      InternalRow updatedRow = new HoodieInternalRow(commitTime, seqId, recordKey,
+      InternalRow updatedRow = new HoodieInternalRow(writeCommitTime, seqId, recordKey,
           partitionPath, fileName, row, true);
-
       try {
         fileWriter.writeRow(recordKey, updatedRow);
         // NOTE: To avoid conversion on the hot-path we only convert [[UTF8String]] into [[String]]
         //       in cases when successful records' writes are being tracked
-        writeStatus.markSuccess(writeStatus.isTrackingSuccessfulWrites() ? recordKey.toString() : null);
+        HoodieRecordDelegate recordDelegate = writeStatus.isTrackingSuccessfulWrites()
+            ? HoodieRecordDelegate.create(recordKey.toString(), partitionPath.toString(), null, newRecordLocation) : null;
+        writeStatus.markSuccess(recordDelegate, Option.empty());
       } catch (Exception t) {
-        writeStatus.markFailure(recordKey.toString(), t);
+        writeStatus.markFailure(recordKey.toString(), partitionPath.toString(), t);
       }
     } catch (Exception e) {
       writeStatus.setGlobalError(e);
@@ -179,7 +207,8 @@ public class HoodieRowCreateHandle implements Serializable {
       // TODO make sure writing w/ and w/o meta fields is consistent (currently writing w/o
       //      meta-fields would fail if any record will, while when writing w/ meta-fields it won't)
       fileWriter.writeRow(row);
-      writeStatus.markSuccess();
+      // when metafields disabled, we do not need to track individual records
+      writeStatus.markSuccess((HoodieRecordDelegate) null, Option.empty());
     } catch (Exception e) {
       writeStatus.setGlobalError(e);
       throw new HoodieException("Exception thrown while writing spark InternalRows to file ", e);
@@ -194,12 +223,12 @@ public class HoodieRowCreateHandle implements Serializable {
   }
 
   /**
-   * Closes the {@link HoodieRowCreateHandle} and returns an instance of {@link HoodieInternalWriteStatus} containing the stats and
+   * Closes the {@link HoodieRowCreateHandle} and returns an instance of {@link WriteStatus} containing the stats and
    * status of the writes to this handle.
    *
-   * @return the {@link HoodieInternalWriteStatus} containing the stats and status of the writes to this handle.
+   * @return the {@link WriteStatus} containing the stats and status of the writes to this handle.
    */
-  public HoodieInternalWriteStatus close() throws IOException {
+  public WriteStatus close() throws IOException {
     fileWriter.close();
     HoodieWriteStat stat = writeStatus.getStat();
     stat.setPartitionPath(partitionPath);
@@ -212,7 +241,10 @@ public class HoodieRowCreateHandle implements Serializable {
     long fileSizeInBytes = FSUtils.getFileSize(table.getMetaClient().getFs(), path);
     stat.setTotalWriteBytes(fileSizeInBytes);
     stat.setFileSizeInBytes(fileSizeInBytes);
-    stat.setTotalWriteErrors(writeStatus.getFailedRowsSize());
+    stat.setTotalWriteErrors(writeStatus.getTotalErrorRecords());
+    for (Pair<HoodieRecordDelegate, Throwable> pair : writeStatus.getFailedRecords()) {
+      LOG.error("Failed to write {}", pair.getLeft(), pair.getRight());
+    }
     HoodieWriteStat.RuntimeStats runtimeStats = new HoodieWriteStat.RuntimeStats();
     runtimeStats.setTotalCreateTime(currTimer.endTimer());
     stat.setRuntimeStats(runtimeStats);
