@@ -166,7 +166,8 @@ public class TestDataValidationCheckForLogCompactionActions extends HoodieClient
       }
       curr++;
     }
-
+    mainTable.client.close();
+    experimentTable.client.close();
   }
 
   private void verifyRecords(TestTableContents mainTable, TestTableContents experimentTable) {
@@ -377,7 +378,7 @@ public class TestDataValidationCheckForLogCompactionActions extends HoodieClient
     // Create logcompaction client.
     HoodieWriteConfig logCompactionConfig = HoodieWriteConfig.newBuilder().withProps(config2.getProps())
         .withCompactionConfig(HoodieCompactionConfig.newBuilder()
-            .withLogCompactionBlocksThreshold("2").build())
+            .withLogCompactionBlocksThreshold(2).build())
         .build();
     SparkRDDWriteClient logCompactionClient = new SparkRDDWriteClient(context, logCompactionConfig);
 
