@@ -252,60 +252,61 @@ public class TestHoodieMergeOnReadTable extends SparkClientFunctionalTestHarness
       // Write them to corresponding avro logfiles
       metaClient = HoodieTableMetaClient.reload(metaClient);
 
-      HoodieTableMetadataWriter metadataWriter = SparkHoodieBackedTableMetadataWriter.create(
-          writeClient.getEngineContext().getHadoopConf().get(), config, writeClient.getEngineContext());
-      HoodieSparkWriteableTestTable testTable = HoodieSparkWriteableTestTable
-          .of(metaClient, HoodieTestDataGenerator.AVRO_SCHEMA_WITH_METADATA_FIELDS, metadataWriter);
+      try (HoodieTableMetadataWriter metadataWriter = SparkHoodieBackedTableMetadataWriter.create(
+          writeClient.getEngineContext().getHadoopConf().get(), config, writeClient.getEngineContext())) {
+        HoodieSparkWriteableTestTable testTable = HoodieSparkWriteableTestTable
+            .of(metaClient, HoodieTestDataGenerator.AVRO_SCHEMA_WITH_METADATA_FIELDS, metadataWriter);
 
-      Set<String> allPartitions = updatedRecords.stream()
-          .map(record -> record.getPartitionPath())
-          .collect(Collectors.groupingBy(partitionPath -> partitionPath))
-          .keySet();
-      assertEquals(allPartitions.size(), testTable.listAllBaseFiles().length);
+        Set<String> allPartitions = updatedRecords.stream()
+            .map(record -> record.getPartitionPath())
+            .collect(Collectors.groupingBy(partitionPath -> partitionPath))
+            .keySet();
+        assertEquals(allPartitions.size(), testTable.listAllBaseFiles().length);
 
-      // Verify that all data file has one log file
-      HoodieTable table = HoodieSparkTable.create(config, context(), metaClient);
-      for (String partitionPath : dataGen.getPartitionPaths()) {
-        List<FileSlice> groupedLogFiles =
-            table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
-        for (FileSlice fileSlice : groupedLogFiles) {
-          assertEquals(1, fileSlice.getLogFiles().count(),
-              "There should be 1 log file written for the latest data file - " + fileSlice);
+        // Verify that all data file has one log file
+        HoodieTable table = HoodieSparkTable.create(config, context(), metaClient);
+        for (String partitionPath : dataGen.getPartitionPaths()) {
+          List<FileSlice> groupedLogFiles =
+              table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
+          for (FileSlice fileSlice : groupedLogFiles) {
+            assertEquals(1, fileSlice.getLogFiles().count(),
+                "There should be 1 log file written for the latest data file - " + fileSlice);
+          }
         }
-      }
 
-      // Do a compaction
-      String compactionInstantTime = writeClient.scheduleCompaction(Option.empty()).get().toString();
-      HoodieWriteMetadata<JavaRDD<WriteStatus>> result = writeClient.compact(compactionInstantTime);
+        // Do a compaction
+        String compactionInstantTime = writeClient.scheduleCompaction(Option.empty()).get().toString();
+        HoodieWriteMetadata<JavaRDD<WriteStatus>> result = writeClient.compact(compactionInstantTime);
 
-      // Verify that recently written compacted data file has no log file
-      metaClient = HoodieTableMetaClient.reload(metaClient);
-      table = HoodieSparkTable.create(config, context(), metaClient);
-      HoodieActiveTimeline timeline = metaClient.getActiveTimeline();
+        // Verify that recently written compacted data file has no log file
+        metaClient = HoodieTableMetaClient.reload(metaClient);
+        table = HoodieSparkTable.create(config, context(), metaClient);
+        HoodieActiveTimeline timeline = metaClient.getActiveTimeline();
 
-      assertTrue(HoodieTimeline
-              .compareTimestamps(timeline.lastInstant().get().getTimestamp(), HoodieTimeline.GREATER_THAN, newCommitTime),
-          "Compaction commit should be > than last insert");
+        assertTrue(HoodieTimeline
+                .compareTimestamps(timeline.lastInstant().get().getTimestamp(), HoodieTimeline.GREATER_THAN, newCommitTime),
+            "Compaction commit should be > than last insert");
 
-      for (String partitionPath : dataGen.getPartitionPaths()) {
-        List<FileSlice> groupedLogFiles =
-            table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
-        for (FileSlice slice : groupedLogFiles) {
-          assertEquals(0, slice.getLogFiles().count(), "After compaction there should be no log files visible on a full view");
+        for (String partitionPath : dataGen.getPartitionPaths()) {
+          List<FileSlice> groupedLogFiles =
+              table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
+          for (FileSlice slice : groupedLogFiles) {
+            assertEquals(0, slice.getLogFiles().count(), "After compaction there should be no log files visible on a full view");
+          }
+          assertTrue(result.getCommitMetadata().get().getWritePartitionPaths().stream().anyMatch(part -> part.contentEquals(partitionPath)));
         }
-        assertTrue(result.getCommitMetadata().get().getWritePartitionPaths().stream().anyMatch(part -> part.contentEquals(partitionPath)));
-      }
 
-      // Check the entire dataset has all records still
-      String[] fullPartitionPaths = new String[dataGen.getPartitionPaths().length];
-      for (int i = 0; i < fullPartitionPaths.length; i++) {
-        fullPartitionPaths[i] = String.format("%s/%s/*", basePath(), dataGen.getPartitionPaths()[i]);
-      }
-      Dataset<Row> actual = HoodieClientTestUtils.read(jsc(), basePath(), sqlContext(), fs(), fullPartitionPaths);
-      List<Row> rows = actual.collectAsList();
-      assertEquals(updatedRecords.size(), rows.size());
-      for (Row row : rows) {
-        assertEquals(row.getAs(HoodieRecord.COMMIT_TIME_METADATA_FIELD), newCommitTime);
+        // Check the entire dataset has all records still
+        String[] fullPartitionPaths = new String[dataGen.getPartitionPaths().length];
+        for (int i = 0; i < fullPartitionPaths.length; i++) {
+          fullPartitionPaths[i] = String.format("%s/%s/*", basePath(), dataGen.getPartitionPaths()[i]);
+        }
+        Dataset<Row> actual = HoodieClientTestUtils.read(jsc(), basePath(), sqlContext(), fs(), fullPartitionPaths);
+        List<Row> rows = actual.collectAsList();
+        assertEquals(updatedRecords.size(), rows.size());
+        for (Row row : rows) {
+          assertEquals(row.getAs(HoodieRecord.COMMIT_TIME_METADATA_FIELD), newCommitTime);
+        }
       }
     }
   }
@@ -360,50 +361,51 @@ public class TestHoodieMergeOnReadTable extends SparkClientFunctionalTestHarness
       // Write them to corresponding avro logfiles
       metaClient = HoodieTableMetaClient.reload(metaClient);
 
-      HoodieTableMetadataWriter metadataWriter = SparkHoodieBackedTableMetadataWriter.create(
-          writeClient.getEngineContext().getHadoopConf().get(), config, writeClient.getEngineContext());
-      HoodieSparkWriteableTestTable testTable = HoodieSparkWriteableTestTable
-          .of(metaClient, HoodieTestDataGenerator.AVRO_SCHEMA_WITH_METADATA_FIELDS, metadataWriter);
+      try (HoodieTableMetadataWriter metadataWriter = SparkHoodieBackedTableMetadataWriter.create(
+          writeClient.getEngineContext().getHadoopConf().get(), config, writeClient.getEngineContext())) {
+        HoodieSparkWriteableTestTable testTable = HoodieSparkWriteableTestTable
+            .of(metaClient, HoodieTestDataGenerator.AVRO_SCHEMA_WITH_METADATA_FIELDS, metadataWriter);
 
-      Set<String> allPartitions = updatedRecords.stream()
-          .map(record -> record.getPartitionPath())
-          .collect(Collectors.groupingBy(partitionPath -> partitionPath))
-          .keySet();
-      assertEquals(allPartitions.size(), testTable.listAllBaseFiles().length);
+        Set<String> allPartitions = updatedRecords.stream()
+            .map(record -> record.getPartitionPath())
+            .collect(Collectors.groupingBy(partitionPath -> partitionPath))
+            .keySet();
+        assertEquals(allPartitions.size(), testTable.listAllBaseFiles().length);
 
-      // Verify that all data file has one log file
-      HoodieTable table = HoodieSparkTable.create(config, context(), metaClient);
-      for (String partitionPath : dataGen.getPartitionPaths()) {
-        List<FileSlice> groupedLogFiles =
-            table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
-        for (FileSlice fileSlice : groupedLogFiles) {
-          assertEquals(2, fileSlice.getLogFiles().count(),
-              "There should be 1 log file written for the latest data file - " + fileSlice);
+        // Verify that all data file has one log file
+        HoodieTable table = HoodieSparkTable.create(config, context(), metaClient);
+        for (String partitionPath : dataGen.getPartitionPaths()) {
+          List<FileSlice> groupedLogFiles =
+              table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
+          for (FileSlice fileSlice : groupedLogFiles) {
+            assertEquals(2, fileSlice.getLogFiles().count(),
+                "There should be 1 log file written for the latest data file - " + fileSlice);
+          }
         }
-      }
 
-      // Do a log compaction
-      String logCompactionInstantTime = writeClient.scheduleLogCompaction(Option.empty()).get().toString();
-      HoodieWriteMetadata<JavaRDD<WriteStatus>> result = writeClient.logCompact(logCompactionInstantTime);
+        // Do a log compaction
+        String logCompactionInstantTime = writeClient.scheduleLogCompaction(Option.empty()).get().toString();
+        HoodieWriteMetadata<JavaRDD<WriteStatus>> result = writeClient.logCompact(logCompactionInstantTime);
 
-      // Verify that recently written compacted data file has no log file
-      metaClient = HoodieTableMetaClient.reload(metaClient);
-      table = HoodieSparkTable.create(config, context(), metaClient);
-      HoodieActiveTimeline timeline = metaClient.getActiveTimeline();
+        // Verify that recently written compacted data file has no log file
+        metaClient = HoodieTableMetaClient.reload(metaClient);
+        table = HoodieSparkTable.create(config, context(), metaClient);
+        HoodieActiveTimeline timeline = metaClient.getActiveTimeline();
 
-      assertTrue(HoodieTimeline
-              .compareTimestamps(timeline.lastInstant().get().getTimestamp(), HoodieTimeline.GREATER_THAN, newCommitTime),
-          "Compaction commit should be > than last insert");
+        assertTrue(HoodieTimeline
+                .compareTimestamps(timeline.lastInstant().get().getTimestamp(), HoodieTimeline.GREATER_THAN, newCommitTime),
+            "Compaction commit should be > than last insert");
 
-      for (String partitionPath : dataGen.getPartitionPaths()) {
-        List<FileSlice> fileSlices =
-            table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
-        assertEquals(1, fileSlices.size());
-        for (FileSlice slice : fileSlices) {
-          assertEquals(3, slice.getLogFiles().count(), "After compaction there will still be one log file.");
-          assertNotNull(slice.getBaseFile(), "Base file is not created by log compaction operation.");
+        for (String partitionPath : dataGen.getPartitionPaths()) {
+          List<FileSlice> fileSlices =
+              table.getSliceView().getLatestFileSlices(partitionPath).collect(Collectors.toList());
+          assertEquals(1, fileSlices.size());
+          for (FileSlice slice : fileSlices) {
+            assertEquals(3, slice.getLogFiles().count(), "After compaction there will still be one log file.");
+            assertNotNull(slice.getBaseFile(), "Base file is not created by log compaction operation.");
+          }
+          assertTrue(result.getCommitMetadata().get().getWritePartitionPaths().stream().anyMatch(part -> part.contentEquals(partitionPath)));
         }
-        assertTrue(result.getCommitMetadata().get().getWritePartitionPaths().stream().anyMatch(part -> part.contentEquals(partitionPath)));
       }
     }
   }
