@@ -21,6 +21,7 @@ package org.apache.hudi.table.functional;
 
 import org.apache.hudi.client.SparkRDDWriteClient;
 import org.apache.hudi.client.WriteStatus;
+import org.apache.hudi.common.config.HoodieStorageConfig;
 import org.apache.hudi.common.model.FileSlice;
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.model.HoodieCommitMetadata;
@@ -46,7 +47,6 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.collection.Pair;
 import org.apache.hudi.config.HoodieCleanConfig;
 import org.apache.hudi.config.HoodieCompactionConfig;
-import org.apache.hudi.common.config.HoodieStorageConfig;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.exception.HoodieIOException;
 import org.apache.hudi.index.HoodieIndex;
@@ -483,24 +483,21 @@ public class TestHoodieSparkMergeOnReadTableRollback extends SparkClientFunction
 
       copyOfRecords.clear();
 
-      // Rollback latest commit first
-      client.restoreToInstant("000", cfg.isMetadataTableEnabled());
+      // Restore to 3rd commit.
+      client.restoreToInstant("003", cfg.isMetadataTableEnabled());
 
-      metaClient = HoodieTableMetaClient.reload(metaClient);
+      metaClient.reloadActiveTimeline();
       allFiles = listAllBaseFilesInPath(hoodieTable);
-      tableView = getHoodieTableFileSystemView(metaClient, metaClient.getCommitTimeline().filterCompletedInstants(), allFiles);
+      tableView = getHoodieTableFileSystemView(metaClient, metaClient.reloadActiveTimeline().getCommitsTimeline().filterCompletedInstants(), allFiles);
       dataFilesToRead = tableView.getLatestBaseFiles();
-      assertFalse(dataFilesToRead.findAny().isPresent());
+      assertFalse(dataFilesToRead.filter(hoodieBaseFile -> hoodieBaseFile.getCommitTime().compareTo("003") > 0).findAny().isPresent());
       TableFileSystemView.SliceView rtView = getHoodieTableFileSystemView(metaClient, metaClient.getCommitTimeline().filterCompletedInstants(), allFiles);
       List<HoodieFileGroup> fileGroups =
           ((HoodieTableFileSystemView) rtView).getAllFileGroups().collect(Collectors.toList());
-      assertTrue(fileGroups.isEmpty());
-
-      // make sure there are no log files remaining
-      assertEquals(0L, ((HoodieTableFileSystemView) rtView).getAllFileGroups()
-          .filter(fileGroup -> fileGroup.getAllRawFileSlices().noneMatch(f -> f.getLogFiles().count() == 0))
-          .count());
+      assertFalse(fileGroups.isEmpty());
 
+      assertFalse(fileGroups.stream().filter(fileGroup -> fileGroup.getAllFileSlices().map(fileSlice
+          -> fileSlice.getBaseInstantTime().compareTo("003") > 0).count() > 0).findAny().isPresent());
     }
   }
 
@@ -556,16 +553,16 @@ public class TestHoodieSparkMergeOnReadTableRollback extends SparkClientFunction
 
       upsertRecords(client, "010", records, dataGen);
 
-      // trigger clean. creating a new client with aggresive cleaner configs so that clean will kick in immediately.
+      // trigger clean. creating a new client with aggressive cleaner configs so that clean will kick in immediately.
       cfgBuilder = getConfigBuilder(false)
           .withCleanConfig(HoodieCleanConfig.newBuilder().retainCommits(1).build())
           // Timeline-server-based markers are not used for multi-rollback tests
           .withMarkersType(MarkerType.DIRECT.name());
       addConfigsForPopulateMetaFields(cfgBuilder, populateMetaFields);
       HoodieWriteConfig cfg1 = cfgBuilder.build();
-      final SparkRDDWriteClient client1 = getHoodieWriteClient(cfg1);
-      client1.clean();
-      client1.close();
+      try (final SparkRDDWriteClient client1 = getHoodieWriteClient(cfg1)) {
+        client1.clean();
+      }
 
       metaClient = HoodieTableMetaClient.reload(metaClient);
       upsertRecords(client, "011", records, dataGen);
@@ -885,31 +882,32 @@ public class TestHoodieSparkMergeOnReadTableRollback extends SparkClientFunction
     HoodieWriteConfig autoCommitFalseCfg = getWriteConfig(false, rollbackUsingMarkers);
     HoodieTestDataGenerator dataGen = new HoodieTestDataGenerator();
 
-    SparkRDDWriteClient client = getHoodieWriteClient(cfg);
-    // commit 1
-    List<HoodieRecord> records = insertRecords(client, dataGen, "001");
-    // commit 2 to create log files
-    List<HoodieRecord> updates1 = updateRecords(client, dataGen, "002", records, metaClient, cfg, true);
+    try (SparkRDDWriteClient client = getHoodieWriteClient(cfg);
+         SparkRDDWriteClient autoCommitFalseClient = getHoodieWriteClient(autoCommitFalseCfg)) {
+      // commit 1
+      List<HoodieRecord> records = insertRecords(client, dataGen, "001");
+      // commit 2 to create log files
+      List<HoodieRecord> updates1 = updateRecords(client, dataGen, "002", records, metaClient, cfg, true);
 
-    // trigger a inflight commit 3 which will be later be rolled back explicitly.
-    SparkRDDWriteClient autoCommitFalseClient = getHoodieWriteClient(autoCommitFalseCfg);
-    List<HoodieRecord> updates2 = updateRecords(autoCommitFalseClient, dataGen, "003", records, metaClient, autoCommitFalseCfg, false);
+      // trigger a inflight commit 3 which will be later be rolled back explicitly.
+      List<HoodieRecord> updates2 = updateRecords(autoCommitFalseClient, dataGen, "003", records, metaClient, autoCommitFalseCfg, false);
 
-    // commit 4 successful (mimic multi-writer scenario)
-    List<HoodieRecord> updates3 = updateRecords(client, dataGen, "004", records, metaClient, cfg, false);
+      // commit 4 successful (mimic multi-writer scenario)
+      List<HoodieRecord> updates3 = updateRecords(client, dataGen, "004", records, metaClient, cfg, false);
 
-    // trigger compaction
-    long numLogFiles = getNumLogFilesInLatestFileSlice(metaClient, cfg, dataGen);
-    doCompaction(autoCommitFalseClient, metaClient, cfg, numLogFiles);
-    long numLogFilesAfterCompaction = getNumLogFilesInLatestFileSlice(metaClient, cfg, dataGen);
-    assertNotEquals(numLogFiles, numLogFilesAfterCompaction);
+      // trigger compaction
+      long numLogFiles = getNumLogFilesInLatestFileSlice(metaClient, cfg, dataGen);
+      doCompaction(autoCommitFalseClient, metaClient, cfg, numLogFiles);
+      long numLogFilesAfterCompaction = getNumLogFilesInLatestFileSlice(metaClient, cfg, dataGen);
+      assertNotEquals(numLogFiles, numLogFilesAfterCompaction);
 
-    // rollback 3rd commit.
-    client.rollback("003");
-    long numLogFilesAfterRollback = getNumLogFilesInLatestFileSlice(metaClient, cfg, dataGen);
-    // lazy rollback should have added the rollback block to previous file slice and not the latest. And so the latest slice's log file count should
-    // remain the same.
-    assertEquals(numLogFilesAfterRollback, numLogFilesAfterCompaction);
+      // rollback 3rd commit.
+      client.rollback("003");
+      long numLogFilesAfterRollback = getNumLogFilesInLatestFileSlice(metaClient, cfg, dataGen);
+      // lazy rollback should have added the rollback block to previous file slice and not the latest. And so the latest slice's log file count should
+      // remain the same.
+      assertEquals(numLogFilesAfterRollback, numLogFilesAfterCompaction);
+    }
   }
 
   private List<HoodieRecord> insertRecords(SparkRDDWriteClient client, HoodieTestDataGenerator dataGen, String commitTime) {
