@@ -207,7 +207,7 @@ public class HoodieCleanerTestBase extends HoodieClientTestBase {
     });
     HoodieCommitMetadata commitMeta = generateCommitMetadata(instantTime, partToFileIds);
     metadataWriter.performTableServices(Option.of(instantTime));
-    metadataWriter.update(commitMeta, context.emptyHoodieData(), instantTime);
+    metadataWriter.updateFromWriteStatuses(commitMeta, context.emptyHoodieData(), instantTime);
     metaClient.getActiveTimeline().saveAsComplete(
         new HoodieInstant(HoodieInstant.State.INFLIGHT, HoodieTimeline.COMMIT_ACTION, instantTime),
         Option.of(commitMeta.toJsonString().getBytes(StandardCharsets.UTF_8)));
