@@ -18,13 +18,17 @@
 
 package org.apache.hudi.common.table.timeline;
 
+import org.apache.hudi.common.util.StringUtils;
 import org.apache.hadoop.fs.FileStatus;
 
 import java.io.Serializable;
 import java.util.Comparator;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Objects;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 /**
  * A Hoodie Instant represents a action done on a hoodie table. All actions start with a inflight instant and then
@@ -34,6 +38,15 @@ import java.util.Objects;
  */
 public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
 
+  // Instant like 20230104152218702.commit.request, 20230104152218702.inflight
+  private static final Pattern NAME_FORMAT =
+      Pattern.compile("^(\\d+)(\\.\\w+)(\\.\\D+)?$");
+
+  private static final String DELIMITER = ".";
+
+  private static final String FILE_NAME_FORMAT_ERROR =
+      "The provided file name %s does not conform to the required format";
+
   /**
    * A COMPACTION action eventually becomes COMMIT when completed. So, when grouping instants
    * for state transitions, this needs to be taken into account
@@ -46,14 +59,35 @@ public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
   public static final Comparator<HoodieInstant> COMPARATOR = Comparator.comparing(HoodieInstant::getTimestamp)
       .thenComparing(ACTION_COMPARATOR).thenComparing(HoodieInstant::getState);
 
+  public static final Comparator<HoodieInstant> STATE_TRANSITION_COMPARATOR =
+      Comparator.comparing(HoodieInstant::getStateTransitionTime)
+          .thenComparing(COMPARATOR);
+
+  public static final String EMPTY_FILE_EXTENSION = "";
+
   public static String getComparableAction(String action) {
     return COMPARABLE_ACTIONS.getOrDefault(action, action);
   }
 
+  public static String extractTimestamp(String fileName) throws IllegalArgumentException {
+    Matcher matcher = NAME_FORMAT.matcher(fileName);
+    if (matcher.find()) {
+      return matcher.group(1);
+    }
+
+    throw new IllegalArgumentException("Failed to retrieve timestamp from name: "
+        + String.format(FILE_NAME_FORMAT_ERROR, fileName));
+  }
+
   public static String getTimelineFileExtension(String fileName) {
     Objects.requireNonNull(fileName);
-    int dotIndex = fileName.indexOf('.');
-    return dotIndex == -1 ? "" : fileName.substring(dotIndex);
+
+    Matcher matcher = NAME_FORMAT.matcher(fileName);
+    if (matcher.find()) {
+      return fileName.substring(matcher.group(1).length());
+    }
+
+    return EMPTY_FILE_EXTENSION;
   }
 
   /**
@@ -70,9 +104,10 @@ public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
     NIL
   }
 
-  private State state = State.COMPLETED;
-  private String action;
-  private String timestamp;
+  private final State state;
+  private final String action;
+  private final String timestamp;
+  private final String stateTransitionTime;
 
   /**
    * Load the instant from the meta FileStatus.
@@ -80,22 +115,27 @@ public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
   public HoodieInstant(FileStatus fileStatus) {
     // First read the instant timestamp. [==>20170101193025<==].commit
     String fileName = fileStatus.getPath().getName();
-    String fileExtension = getTimelineFileExtension(fileName);
-    timestamp = fileName.replace(fileExtension, "");
-
-    // Next read the action for this marker
-    action = fileExtension.replaceFirst(".", "");
-    if (action.equals("inflight")) {
-      // This is to support backwards compatibility on how in-flight commit files were written
-      // General rule is inflight extension is .<action>.inflight, but for commit it is .inflight
-      action = "commit";
-      state = State.INFLIGHT;
-    } else if (action.contains(HoodieTimeline.INFLIGHT_EXTENSION)) {
-      state = State.INFLIGHT;
-      action = action.replace(HoodieTimeline.INFLIGHT_EXTENSION, "");
-    } else if (action.contains(HoodieTimeline.REQUESTED_EXTENSION)) {
-      state = State.REQUESTED;
-      action = action.replace(HoodieTimeline.REQUESTED_EXTENSION, "");
+    Matcher matcher = NAME_FORMAT.matcher(fileName);
+    if (matcher.find()) {
+      timestamp = matcher.group(1);
+      if (matcher.group(2).equals(HoodieTimeline.INFLIGHT_EXTENSION)) {
+        // This is to support backwards compatibility on how in-flight commit files were written
+        // General rule is inflight extension is .<action>.inflight, but for commit it is .inflight
+        action = HoodieTimeline.COMMIT_ACTION;
+        state = State.INFLIGHT;
+      } else {
+        action = matcher.group(2).replaceFirst(DELIMITER, StringUtils.EMPTY_STRING);
+        if (matcher.groupCount() == 3 && matcher.group(3) != null) {
+          state = State.valueOf(matcher.group(3).replaceFirst(DELIMITER, StringUtils.EMPTY_STRING).toUpperCase());
+        } else {
+          // Like 20230104152218702.commit
+          state = State.COMPLETED;
+        }
+      }
+      stateTransitionTime =
+          HoodieInstantTimeGenerator.formatDate(new Date(fileStatus.getModificationTime()));
+    } else {
+      throw new IllegalArgumentException("Failed to construct HoodieInstant: " + String.format(FILE_NAME_FORMAT_ERROR, fileName));
     }
   }
 
@@ -104,12 +144,21 @@ public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
     this.state = isInflight ? State.INFLIGHT : State.COMPLETED;
     this.action = action;
     this.timestamp = timestamp;
+    this.stateTransitionTime = null;
   }
 
   public HoodieInstant(State state, String action, String timestamp) {
     this.state = state;
     this.action = action;
     this.timestamp = timestamp;
+    this.stateTransitionTime = null;
+  }
+
+  public HoodieInstant(State state, String action, String timestamp, String stateTransitionTime) {
+    this.state = state;
+    this.action = action;
+    this.timestamp = timestamp;
+    this.stateTransitionTime = stateTransitionTime;
   }
 
   public boolean isCompleted() {
@@ -214,6 +263,10 @@ public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
     return state;
   }
 
+  public String getStateTransitionTime() {
+    return stateTransitionTime;
+  }
+
   @Override
   public int hashCode() {
     return Objects.hash(state, action, timestamp);
@@ -226,6 +279,8 @@ public class HoodieInstant implements Serializable, Comparable<HoodieInstant> {
 
   @Override
   public String toString() {
-    return "[" + ((isInflight() || isRequested()) ? "==>" : "") + timestamp + "__" + action + "__" + state + "]";
+    return "[" + ((isInflight() || isRequested()) ? "==>" : "")
+        + timestamp + "__" + action + "__" + state
+        + (StringUtils.isNullOrEmpty(stateTransitionTime) ? "" : ("__" + stateTransitionTime)) + "]";
   }
 }
