@@ -21,6 +21,7 @@ package org.apache.hudi.common.util;
 import javax.annotation.Nullable;
 
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.util.Collections;
 import java.util.List;
 import java.util.stream.Collectors;
@@ -31,6 +32,7 @@ import java.util.stream.Stream;
  */
 public class StringUtils {
 
+  public static final char[] HEX_CHAR = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
   public static final String EMPTY_STRING = "";
 
   /**
@@ -86,11 +88,24 @@ public class StringUtils {
   }
 
   public static String toHexString(byte[] bytes) {
-    StringBuilder sb = new StringBuilder(bytes.length * 2);
-    for (byte b : bytes) {
-      sb.append(String.format("%02x", b));
+    return new String(encodeHex(bytes));
+  }
+
+  public static char[] encodeHex(byte[] data) {
+    int l = data.length;
+    char[] out = new char[l << 1];
+    int i = 0;
+
+    for (int var4 = 0; i < l; ++i) {
+      out[var4++] = HEX_CHAR[(240 & data[i]) >>> 4];
+      out[var4++] = HEX_CHAR[15 & data[i]];
     }
-    return sb.toString();
+
+    return out;
+  }
+
+  public static byte[] getUTF8Bytes(String str) {
+    return str.getBytes(StandardCharsets.UTF_8);
   }
 
   public static boolean isNullOrEmpty(String str) {
@@ -158,4 +173,14 @@ public class StringUtils {
     }
     return input.substring(0, i);
   }
+
+  public static String truncate(String str, int headLength, int tailLength) {
+    if (isNullOrEmpty(str) || str.length() <= headLength + tailLength) {
+      return str;
+    }
+    String head = str.substring(0, headLength);
+    String tail = str.substring(str.length() - tailLength);
+
+    return head + "..." + tail;
+  }
 }
