@@ -28,17 +28,15 @@ import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.configuration.FlinkOptions;
-import org.apache.hudi.exception.HoodieIOException;
+import org.apache.hudi.metadata.HoodieTableMetadata;
 import org.apache.hudi.sink.compact.FlinkCompactionConfig;
 import org.apache.hudi.table.HoodieFlinkTable;
 
 import org.apache.avro.Schema;
 import org.apache.flink.configuration.Configuration;
-import org.apache.hadoop.fs.Path;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.io.IOException;
 import java.util.Locale;
 
 /**
@@ -130,7 +128,9 @@ public class CompactionUtil {
    */
   public static void setPreCombineField(Configuration conf, HoodieTableMetaClient metaClient) {
     String preCombineField = metaClient.getTableConfig().getPreCombineField();
-    conf.setString(FlinkOptions.PRECOMBINE_FIELD, preCombineField);
+    if (preCombineField != null) {
+      conf.setString(FlinkOptions.PRECOMBINE_FIELD, preCombineField);
+    }
   }
 
   /**
@@ -139,6 +139,7 @@ public class CompactionUtil {
    * <p>We can improve the code if the changelog mode is set up as table config.
    *
    * @param conf The configuration
+   * @param metaClient The meta client
    */
   public static void inferChangelogMode(Configuration conf, HoodieTableMetaClient metaClient) throws Exception {
     TableSchemaResolver tableSchemaResolver = new TableSchemaResolver(metaClient);
@@ -149,21 +150,17 @@ public class CompactionUtil {
   }
 
   /**
-   * Cleans the metadata file for given instant {@code instant}.
+   * Infers the metadata config based on the existence of metadata folder.
+   *
+   * <p>We can improve the code if the metadata config is set up as table config.
+   *
+   * @param conf The configuration
+   * @param metaClient The meta client
    */
-  public static void cleanInstant(HoodieTableMetaClient metaClient, HoodieInstant instant) {
-    Path commitFilePath = new Path(metaClient.getMetaAuxiliaryPath(), instant.getFileName());
-    try {
-      if (metaClient.getFs().exists(commitFilePath)) {
-        boolean deleted = metaClient.getFs().delete(commitFilePath, false);
-        if (deleted) {
-          LOG.info("Removed instant " + instant);
-        } else {
-          throw new HoodieIOException("Could not delete instant " + instant);
-        }
-      }
-    } catch (IOException e) {
-      throw new HoodieIOException("Could not remove requested commit " + commitFilePath, e);
+  public static void inferMetadataConf(Configuration conf, HoodieTableMetaClient metaClient) {
+    String path = HoodieTableMetadata.getMetadataTableBasePath(conf.getString(FlinkOptions.PATH));
+    if (!StreamerUtil.tableExists(path, metaClient.getHadoopConf())) {
+      conf.setBoolean(FlinkOptions.METADATA_ENABLED, false);
     }
   }
 
