@@ -18,7 +18,7 @@
 
 package org.apache.hudi.adapter;
 
-import org.apache.flink.streaming.api.operators.MailboxExecutor;
+import org.apache.flink.api.common.operators.MailboxExecutor;
 import org.apache.flink.util.function.ThrowingRunnable;
 
 /**
