@@ -18,9 +18,6 @@
 package org.apache.spark.sql.hudi.command.procedures
 
 import org.apache.hudi.HoodieCLIUtils
-import org.apache.hudi.client.SparkRDDWriteClient
-import org.apache.hudi.client.common.HoodieSparkEngineContext
-import org.apache.hudi.common.model.HoodieRecordPayload
 import org.apache.hudi.config.{HoodieIndexConfig, HoodieWriteConfig}
 import org.apache.hudi.exception.HoodieException
 import org.apache.hudi.index.HoodieIndex.IndexType
@@ -30,18 +27,11 @@ import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.types._
 
 abstract class BaseProcedure extends Procedure {
-  val INVALID_ARG_INDEX: Int = -1
-
   val spark: SparkSession = SparkSession.active
   val jsc = new JavaSparkContext(spark.sparkContext)
 
   protected def sparkSession: SparkSession = spark
 
-  protected def createHoodieClient(jsc: JavaSparkContext, basePath: String): SparkRDDWriteClient[_ <: HoodieRecordPayload[_ <: AnyRef]] = {
-    val config = getWriteConfig(basePath)
-    new SparkRDDWriteClient(new HoodieSparkEngineContext(jsc), config)
-  }
-
   protected def getWriteConfig(basePath: String): HoodieWriteConfig = {
     HoodieWriteConfig.newBuilder
       .withPath(basePath)
@@ -49,41 +39,31 @@ abstract class BaseProcedure extends Procedure {
       .build
   }
 
-  protected def checkArgs(target: Array[ProcedureParameter], args: ProcedureArgs): Unit = {
-    val internalRow = args.internalRow
-    for (i <- target.indices) {
-      if (target(i).required) {
-        var argsIndex: Integer = null
-        if (args.isNamedArgs) {
-          argsIndex = getArgsIndex(target(i).name, args)
-        } else {
-          argsIndex = getArgsIndex(i.toString, args)
-        }
-        assert(-1 != argsIndex && internalRow.get(argsIndex, target(i).dataType) != null,
-          s"Argument: ${target(i).name} is required")
-      }
+  protected def getParamKey(parameter: ProcedureParameter, isNamedArgs: Boolean): String = {
+    if (isNamedArgs) {
+      parameter.name
+    } else {
+      parameter.index.toString
     }
   }
 
-  protected def getArgsIndex(key: String, args: ProcedureArgs): Integer = {
-    args.map.getOrDefault(key, INVALID_ARG_INDEX)
+  protected def checkArgs(parameters: Array[ProcedureParameter], args: ProcedureArgs): Unit = {
+    for (parameter <- parameters) {
+      if (parameter.required) {
+        val paramKey = getParamKey(parameter, args.isNamedArgs)
+        assert(args.map.containsKey(paramKey) &&
+          args.internalRow.get(args.map.get(paramKey), parameter.dataType) != null,
+          s"Argument: ${parameter.name} is required")
+      }
+    }
   }
 
   protected def getArgValueOrDefault(args: ProcedureArgs, parameter: ProcedureParameter): Option[Any] = {
-    var argsIndex: Int = INVALID_ARG_INDEX
-    if (args.isNamedArgs) {
-      argsIndex = getArgsIndex(parameter.name, args)
+    val paramKey = getParamKey(parameter, args.isNamedArgs)
+    if (args.map.containsKey(paramKey)) {
+      Option.apply(getInternalRowValue(args.internalRow, args.map.get(paramKey), parameter.dataType))
     } else {
-      argsIndex = getArgsIndex(parameter.index.toString, args)
-    }
-
-    if (argsIndex.equals(INVALID_ARG_INDEX)) {
-      parameter.default match {
-        case option: Option[Any] => option
-        case _ => Option.apply(parameter.default)
-      }
-    } else {
-      Option.apply(getInternalRowValue(args.internalRow, argsIndex, parameter.dataType))
+      Option.apply(parameter.default)
     }
   }
 
