@@ -26,16 +26,14 @@ import org.apache.hudi.common.util.Option
 import org.apache.hudi.config.HoodieWriteConfig.ROLLBACK_USING_MARKERS_ENABLE
 import org.apache.hudi.exception.HoodieException
 import org.apache.spark.sql.Row
-import org.apache.spark.sql.catalyst.TableIdentifier
-import org.apache.spark.sql.catalyst.catalog.HoodieCatalogTable
 import org.apache.spark.sql.types.{DataTypes, Metadata, StructField, StructType}
 
 import java.util.function.Supplier
 
 class RollbackToInstantTimeProcedure extends BaseProcedure with ProcedureBuilder {
   private val PARAMETERS = Array[ProcedureParameter](
-    ProcedureParameter.required(0, "table", DataTypes.StringType, None),
-    ProcedureParameter.required(1, "instant_time", DataTypes.StringType, None))
+    ProcedureParameter.required(0, "table", DataTypes.StringType),
+    ProcedureParameter.required(1, "instant_time", DataTypes.StringType))
 
   private val OUTPUT_TYPE = new StructType(Array[StructField](
     StructField("rollback_result", DataTypes.BooleanType, nullable = true, Metadata.empty))
@@ -55,7 +53,7 @@ class RollbackToInstantTimeProcedure extends BaseProcedure with ProcedureBuilder
     val basePath = hoodieCatalogTable.tableLocation
     var client: SparkRDDWriteClient[_] = null
     try {
-      client = createHoodieClient(jsc, basePath)
+      client = HoodieCLIUtils.createHoodieWriteClient(sparkSession, basePath, Map.empty, scala.Option(table))
       client.getConfig.setValue(ROLLBACK_USING_MARKERS_ENABLE, "false")
       val config = getWriteConfig(basePath)
       val metaClient = HoodieTableMetaClient.builder
@@ -67,13 +65,13 @@ class RollbackToInstantTimeProcedure extends BaseProcedure with ProcedureBuilder
         .build
 
       val activeTimeline = metaClient.getActiveTimeline
-      val completedTimeline: HoodieTimeline = activeTimeline.getCommitsTimeline.filterCompletedInstants
-      val filteredTimeline = completedTimeline.containsInstant(instantTime)
+      val filteredTimeline = activeTimeline.containsInstant(instantTime)
       if (!filteredTimeline) {
-        throw new HoodieException(s"Commit $instantTime not found in Commits $completedTimeline")
+        throw new HoodieException(s"Commit $instantTime not found in Commits $activeTimeline")
       }
 
       val result = if (client.rollback(instantTime)) true else false
+      spark.catalog.refreshTable(table)
       val outputRow = Row(result)
 
       Seq(outputRow)
