@@ -230,6 +230,8 @@ public class TestSparkConsistentBucketClustering extends HoodieSparkClientTestHa
     } else {
       options.put(HoodieClusteringConfig.PLAN_STRATEGY_SORT_COLUMNS.key(), sortColumn);
     }
+    // TODO: row writer does not support sort for consistent hashing index
+    options.put("hoodie.datasource.write.row.writer.enable", String.valueOf(false));
     setup(128 * 1024 * 1024, options);
 
     writeData(HoodieActiveTimeline.createNewInstantTime(), 500, true);
@@ -254,6 +256,8 @@ public class TestSparkConsistentBucketClustering extends HoodieSparkClientTestHa
       throw new HoodieException("Cannot get comparator: unsupported data type, " + field.schema().getType());
     }
 
+    // Note: If row writer is used, it will throw: https://github.com/apache/hudi/issues/8838
+    // Use #readRecords() instead if row-writer is used in the future
     for (RecordReader recordReader: readers) {
       Object key = recordReader.createKey();
       ArrayWritable writable = (ArrayWritable) recordReader.createValue();
