@@ -18,12 +18,9 @@
 
 package org.apache.hudi
 
-import org.apache.hudi.HoodieUnsafeRDD
 import org.apache.spark.sql.SparkSession
 import org.apache.spark.sql.catalyst.InternalRow
-import org.apache.spark.sql.catalyst.expressions.AttributeReference
 import org.apache.spark.sql.execution.datasources.{FilePartition, FileScanRDD, PartitionedFile}
-import org.apache.spark.sql.types.StructType
 
 class Spark32HoodieFileScanRDD(@transient private val sparkSession: SparkSession,
                                read: PartitionedFile => Iterator[InternalRow],
