@@ -39,8 +39,8 @@ import com.fasterxml.jackson.databind.ObjectMapper;
 import io.javalin.http.Context;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.Collections;
@@ -80,7 +80,7 @@ import static org.apache.hudi.timeline.service.RequestHandler.jsonifyResult;
  * Worker Thread 3               |-------------------------->| writing to MARKERS2
  */
 public class MarkerHandler extends Handler {
-  private static final Logger LOG = LogManager.getLogger(MarkerHandler.class);
+  private static final Logger LOG = LoggerFactory.getLogger(MarkerHandler.class);
 
   private final Registry metricsRegistry;
   // a scheduled executor service to schedule dispatching of marker creation requests
@@ -126,8 +126,8 @@ public class MarkerHandler extends Handler {
     if (dispatchingThreadFuture != null) {
       dispatchingThreadFuture.cancel(true);
     }
-    dispatchingExecutorService.shutdown();
-    batchingExecutorService.shutdown();
+    dispatchingExecutorService.shutdownNow();
+    batchingExecutorService.shutdownNow();
   }
 
   /**
@@ -242,7 +242,7 @@ public class MarkerHandler extends Handler {
 
   private MarkerCreationFuture addMarkerCreationRequestForAsyncProcessing(
       Context context, String markerDir, String markerName) {
-    LOG.info("Request: create marker " + markerDir + " " + markerName);
+    LOG.info("Request: create marker: " + markerName);
     MarkerCreationFuture future = new MarkerCreationFuture(context, markerDir, markerName);
     // Add the future to the list
     MarkerDirState markerDirState = getMarkerDirState(markerDir);
