@@ -25,14 +25,15 @@ import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.ValidationUtils;
 import org.apache.hudi.common.util.collection.ImmutablePair;
 import org.apache.hudi.common.util.collection.Pair;
+import org.apache.hudi.utilities.config.DatePartitionPathSelectorConfig;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
 import org.apache.spark.api.java.JavaSparkContext;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.time.LocalDate;
 import java.time.format.DateTimeFormatter;
@@ -41,15 +42,13 @@ import java.util.Comparator;
 import java.util.List;
 import java.util.stream.Collectors;
 
-import static org.apache.hudi.utilities.sources.helpers.DFSPathSelector.Config.ROOT_INPUT_PATH_PROP;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.DATE_FORMAT;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.DATE_PARTITION_DEPTH;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.DEFAULT_DATE_FORMAT;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.DEFAULT_DATE_PARTITION_DEPTH;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.DEFAULT_LOOKBACK_DAYS;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.DEFAULT_PARTITIONS_LIST_PARALLELISM;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.LOOKBACK_DAYS;
-import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelector.Config.PARTITIONS_LIST_PARALLELISM;
+import static org.apache.hudi.common.util.ConfigUtils.getIntWithAltKeys;
+import static org.apache.hudi.common.util.ConfigUtils.getStringWithAltKeys;
+import static org.apache.hudi.utilities.config.DFSPathSelectorConfig.ROOT_INPUT_PATH;
+import static org.apache.hudi.utilities.config.DatePartitionPathSelectorConfig.DATE_FORMAT;
+import static org.apache.hudi.utilities.config.DatePartitionPathSelectorConfig.DATE_PARTITION_DEPTH;
+import static org.apache.hudi.utilities.config.DatePartitionPathSelectorConfig.LOOKBACK_DAYS;
+import static org.apache.hudi.utilities.config.DatePartitionPathSelectorConfig.PARTITIONS_LIST_PARALLELISM;
 
 /**
  * Custom dfs path selector used to list just the last few days provided there is a date based
@@ -65,11 +64,11 @@ import static org.apache.hudi.utilities.sources.helpers.DatePartitionPathSelecto
  * `<basepath>/<<date-based-partition>/`.
  *
  * <p>The date based partition format can be configured via this property
- * hoodie.deltastreamer.source.dfs.datepartitioned.date.format
+ * hoodie.streamer.source.dfs.datepartitioned.date.format
  */
 public class DatePartitionPathSelector extends DFSPathSelector {
 
-  private static volatile Logger LOG = LogManager.getLogger(DatePartitionPathSelector.class);
+  private static volatile Logger LOG = LoggerFactory.getLogger(DatePartitionPathSelector.class);
 
   private final String dateFormat;
   private final int datePartitionDepth;
@@ -78,23 +77,24 @@ public class DatePartitionPathSelector extends DFSPathSelector {
 
   /** Configs supported. */
   public static class Config {
-    public static final String DATE_FORMAT = "hoodie.deltastreamer.source.dfs.datepartitioned.date.format";
-    public static final String DEFAULT_DATE_FORMAT = "yyyy-MM-dd";
-
-    public static final String DATE_PARTITION_DEPTH =
-        "hoodie.deltastreamer.source.dfs.datepartitioned.selector.depth";
-    public static final int DEFAULT_DATE_PARTITION_DEPTH = 0; // Implies no (date) partition
-
-    public static final String LOOKBACK_DAYS =
-        "hoodie.deltastreamer.source.dfs.datepartitioned.selector.lookback.days";
-    public static final int DEFAULT_LOOKBACK_DAYS = 2;
-
-    public static final String CURRENT_DATE =
-        "hoodie.deltastreamer.source.dfs.datepartitioned.selector.currentdate";
-
-    public static final String PARTITIONS_LIST_PARALLELISM =
-        "hoodie.deltastreamer.source.dfs.datepartitioned.selector.parallelism";
-    public static final int DEFAULT_PARTITIONS_LIST_PARALLELISM = 20;
+    @Deprecated
+    public static final String DATE_FORMAT = DatePartitionPathSelectorConfig.DATE_FORMAT.key();
+    @Deprecated
+    public static final String DEFAULT_DATE_FORMAT = DatePartitionPathSelectorConfig.DATE_FORMAT.defaultValue();
+    @Deprecated
+    public static final String DATE_PARTITION_DEPTH = DatePartitionPathSelectorConfig.DATE_PARTITION_DEPTH.key();
+    @Deprecated
+    public static final int DEFAULT_DATE_PARTITION_DEPTH = DatePartitionPathSelectorConfig.DATE_PARTITION_DEPTH.defaultValue(); // Implies no (date) partition
+    @Deprecated
+    public static final String LOOKBACK_DAYS = DatePartitionPathSelectorConfig.LOOKBACK_DAYS.key();
+    @Deprecated
+    public static final int DEFAULT_LOOKBACK_DAYS = DatePartitionPathSelectorConfig.LOOKBACK_DAYS.defaultValue();
+    @Deprecated
+    public static final String CURRENT_DATE = DatePartitionPathSelectorConfig.CURRENT_DATE.key();
+    @Deprecated
+    public static final String PARTITIONS_LIST_PARALLELISM = DatePartitionPathSelectorConfig.PARTITIONS_LIST_PARALLELISM.key();
+    @Deprecated
+    public static final int DEFAULT_PARTITIONS_LIST_PARALLELISM = DatePartitionPathSelectorConfig.PARTITIONS_LIST_PARALLELISM.defaultValue();
   }
 
   public DatePartitionPathSelector(TypedProperties props, Configuration hadoopConf) {
@@ -103,10 +103,10 @@ public class DatePartitionPathSelector extends DFSPathSelector {
      * datePartitionDepth = 0 is same as basepath and there is no partition. In which case
      * this path selector would be a no-op and lists all paths under the table basepath.
      */
-    dateFormat = props.getString(DATE_FORMAT, DEFAULT_DATE_FORMAT);
-    datePartitionDepth = props.getInteger(DATE_PARTITION_DEPTH, DEFAULT_DATE_PARTITION_DEPTH);
-    numPrevDaysToList = props.getInteger(LOOKBACK_DAYS, DEFAULT_LOOKBACK_DAYS);
-    partitionsListParallelism = props.getInteger(PARTITIONS_LIST_PARALLELISM, DEFAULT_PARTITIONS_LIST_PARALLELISM);
+    dateFormat = getStringWithAltKeys(props, DATE_FORMAT, true);
+    datePartitionDepth = getIntWithAltKeys(props, DATE_PARTITION_DEPTH);
+    numPrevDaysToList = getIntWithAltKeys(props, LOOKBACK_DAYS);
+    partitionsListParallelism = getIntWithAltKeys(props, PARTITIONS_LIST_PARALLELISM);
   }
 
   @Override
@@ -114,12 +114,13 @@ public class DatePartitionPathSelector extends DFSPathSelector {
                                                                              Option<String> lastCheckpointStr,
                                                                              long sourceLimit) {
     // If not specified the current date is assumed by default.
-    LocalDate currentDate = LocalDate.parse(props.getString(Config.CURRENT_DATE, LocalDate.now().toString()));
+    LocalDate currentDate = LocalDate.parse(
+        getStringWithAltKeys(props, DatePartitionPathSelectorConfig.CURRENT_DATE, LocalDate.now().toString()));
 
     // obtain all eligible files under root folder.
     LOG.info(
         "Root path => "
-            + props.getString(ROOT_INPUT_PATH_PROP)
+            + getStringWithAltKeys(props, ROOT_INPUT_PATH)
             + " source limit => "
             + sourceLimit
             + " depth of day partition => "
@@ -131,7 +132,8 @@ public class DatePartitionPathSelector extends DFSPathSelector {
     long lastCheckpointTime = lastCheckpointStr.map(Long::parseLong).orElse(Long.MIN_VALUE);
     HoodieSparkEngineContext context = new HoodieSparkEngineContext(sparkContext);
     SerializableConfiguration serializedConf = new SerializableConfiguration(fs.getConf());
-    List<String> prunedPartitionPaths = pruneDatePartitionPaths(context, fs, props.getString(ROOT_INPUT_PATH_PROP), currentDate);
+    List<String> prunedPartitionPaths = pruneDatePartitionPaths(
+        context, fs, getStringWithAltKeys(props, ROOT_INPUT_PATH), currentDate);
 
     List<FileStatus> eligibleFiles = context.flatMap(prunedPartitionPaths,
         path -> {
