@@ -36,6 +36,7 @@ import org.apache.spark.sql.functions;
 import org.apache.spark.sql.types.DataTypes;
 import org.apache.spark.storage.StorageLevel;
 import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeAll;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
@@ -64,6 +65,11 @@ public class TestJdbcSource extends UtilitiesTestBase {
   private static final HoodieTestDataGenerator DATA_GENERATOR = new HoodieTestDataGenerator();
   private static Connection connection;
 
+  @BeforeAll
+  public static void beforeAll() throws Exception {
+    UtilitiesTestBase.initTestServices(false, false, false);
+  }
+
   @BeforeEach
   public void setup() throws Exception {
     super.setup();
